@@ -1,15 +1,15 @@
-#' Reformat R code while preserving blank lines and comments
+#' Reformat R code
 #'
-#' Read R code from a file or the clipboard and reformat it. That this function
-#' preserves blank lines and comments is a different behavior from that of
-#' \code{\link{parse}()} and \code{\link{deparse}()}. This function can also
-#' replace \code{=} with \code{<-} where \code{=} means assignment, and
-#' re-indent code with a specified number of spaces.
+#' Read R code from a file or the clipboard and reformat it. This function is
+#' based on \code{\link{parse}()} and \code{\link{deparse}()}, but it does
+#' several other things, such as preserving blank lines and comments,
+#' substituting the assignment operator \code{=} with \code{<-}, and
+#' re-indenting code with a specified number of spaces.
 #'
-#' Value of the argument \code{width.cutoff} wrapped in \code{\link{I}()} (e.g.,
-#' \code{I(60)}) will be treated as the \emph{upper bound} of the line width.
-#' The corresponding argument to \code{deparse()} is a lower bound, so the
-#' function will perform a binary search for a width value that can make
+#' A value of the argument \code{width.cutoff} wrapped in \code{\link{I}()}
+#' (e.g., \code{I(60)}) will be treated as the \emph{upper bound} of the line
+#' width. The corresponding argument to \code{deparse()} is a lower bound, so
+#' the function will perform a binary search for a width value that can make
 #' \code{deparse()} return code with line width smaller than or equal to the
 #' \code{width.cutoff} value. If the search fails, a warning will signal,
 #' suppressible by global option \code{options(formatR.width.warning = FALSE)}.
@@ -17,7 +17,10 @@
 #'   the clipboard).
 #' @param comment Whether to keep comments.
 #' @param blank Whether to keep blank lines.
-#' @param arrow Whether to replace the assign operator \code{=} with \code{<-}.
+#' @param arrow Whether to substitute the assignment operator \code{=} with
+#'   \code{<-}.
+#' @param pipe Whether to substitute the \pkg{magrittr} pipe \code{\%>\%} with
+#'   R's native pipe operator \code{|>}.
 #' @param brace.newline Whether to put the left brace \code{\{} to a new line.
 #' @param indent Number of spaces to indent the code.
 #' @param wrap Whether to wrap comments to the linewidth determined by
@@ -26,6 +29,9 @@
 #'   length is at or over this number, the function will try to break it into a
 #'   new line. In other words, this is the \emph{lower bound} of the line width.
 #'   See \sQuote{Details} if an upper bound is desired instead.
+#' @param args.newline Whether to start the arguments of a function call on a
+#'   new line instead of after the function name and \code{(} when the arguments
+#'   cannot fit one line.
 #' @param output Whether to output to the console or a file using
 #'   \code{\link{cat}()}.
 #' @param text An alternative way to specify the input: if \code{NULL}, the
@@ -51,10 +57,12 @@ tidy_source = function(
   source = 'clipboard', comment = getOption('formatR.comment', TRUE),
   blank = getOption('formatR.blank', TRUE),
   arrow = getOption('formatR.arrow', FALSE),
+  pipe = getOption('formatR.pipe', FALSE),
   brace.newline = getOption('formatR.brace.newline', FALSE),
   indent = getOption('formatR.indent', 4),
   wrap = getOption('formatR.wrap', TRUE),
   width.cutoff = getOption('formatR.width', getOption('width')),
+  args.newline = getOption('formatR.args.newline', FALSE),
   output = TRUE, text = NULL, ...
 ) {
   if (is.null(text)) {
@@ -83,12 +91,12 @@ tidy_source = function(
   if (width.cutoff < 20) width.cutoff[1] = 20
   # insert enough spaces into infix operators such as %>% so the lines can be
   # broken after the operators
-  spaces = paste(rep(' ', width.cutoff), collapse = '')
-  if (comment) text = mask_comments(text, blank, wrap, spaces)
+  spaces = rep_chars(width.cutoff)
+  text = mask_comments(text, comment, blank, wrap, arrow, pipe, args.newline, spaces)
   text.mask = tidy_block(
-    text, width.cutoff, arrow && length(grep('=', text)), indent, brace.newline, wrap
+    text, width.cutoff, rep_chars(indent), brace.newline, wrap, args.newline, spaces
   )
-  text.tidy = if (comment) unmask_source(text.mask, spaces) else text.mask
+  text.tidy = if (comment) unmask_source(text.mask) else text.mask
   # restore new lines in the beginning and end
   if (blank) text.tidy = c(rep('', n1), text.tidy, rep('', n2))
   if (output) cat(text.tidy, sep = '\n', ...)
@@ -111,7 +119,10 @@ blank.comment2 = paste0('^\\s*', gsub('\\(', '\\\\(', blank.comment), '\\s*$')
 # first, perform a (semi-)binary search to find the greatest cutoff width such
 # that the width of the longest line <= `width`; if the search fails, use
 # brute-force to try all possible widths
-deparse2 = function(expr, width, warn = getOption('formatR.width.warning', TRUE)) {
+deparse2 = function(
+  expr, width, spaces = '', indent = '    ',
+  warn = getOption('formatR.width.warning', TRUE)
+) {
   wmin = 20  # if deparse() can't manage it with width.cutoff <= 20, issue a warning
   wmax = min(500, width + 10)  # +10 because a larger width may result in smaller actual width
 
@@ -125,12 +136,14 @@ deparse2 = function(expr, width, warn = getOption('formatR.width.warning', TRUE)
     i = as.character(w)
     if (!is.na(x <- k[i])) return(x)
     x = deparse(expr, w)
-    x = gsub('\\s+$', '', x)
+    x = trimws(x, 'right')
     d[[i]] <<- x
     x2 = grep(pat.comment, x, invert = TRUE, value = TRUE)  # don't check comments
     x2 = gsub(pat.infix, '\\1\\2\\3', x2)  # remove extra spaces in %>% operators
-    x2 = restore_pipe(x2)
-    p[[i]] <<- x2[nchar(x2, type = 'width') > width]
+    x2 = restore_infix(x2)
+    x2 = reindent_lines(x2, indent)
+    x2 = restore_arg_breaks(x2, width, spaces, indent, split = TRUE)
+    p[[i]] <<- x2[exceed_width(x2, width)]
     k[i] <<- length(p[[i]]) == 0
   }
 
@@ -160,45 +173,47 @@ deparse2 = function(expr, width, warn = getOption('formatR.width.warning', TRUE)
 
 # wrapper around parse() and deparse()
 tidy_block = function(
-  text, width = getOption('width'), arrow = FALSE, indent = 4,
-  brace.newline = FALSE, wrap = TRUE
+  text, width = getOption('width'), indent = '    ',
+  brace.newline = FALSE, wrap = TRUE, args.newline = FALSE, spaces = rep_chars(width)
 ) {
-  exprs = parse_only(text)
+  exprs = parse_source(text)
   if (length(exprs) == 0) return(character(0))
-  exprs = if (arrow) replace_assignment(exprs) else as.list(exprs)
-  deparse = if (inherits(width, 'AsIs')) deparse2 else base::deparse
-  unlist(lapply(exprs, function(e) {
+  deparse = if (inherits(width, 'AsIs')) {
+    function(x, width) deparse2(x, width, spaces, indent)
+  } else base::deparse
+  unlist(lapply(as.list(exprs), function(e) {
     x = deparse(e, width)
+    x = trimws(x, 'right')
     x = reindent_lines(x, indent)
     # remove white spaces on blank lines
     x = gsub(blank.comment2, '', x)
     x = reflow_comments(x, width, wrap)
     if (brace.newline) x = move_leftbrace(x)
-    x = restore_pipe(x)
-    one_string(x)
+    x = restore_infix(x)
+    x = one_string(x)
+    if (args.newline) x = restore_arg_breaks(x, width, spaces, indent)
+    x
   }))
 }
 
 # Restore the real source code from the masked text
-unmask_source = function(text.mask, spaces) {
-  if (length(text.mask) == 0) return(text.mask)
+unmask_source = function(x) {
+  if (length(x) == 0) return(x)
   m = .env$line_break
-  if (!is.null(m)) text.mask = gsub(m, '\n', text.mask)
+  if (!is.null(m)) x = gsub(m, '\n', x)
   # if the comments were separated into the next line, then remove '\n' after
   # the identifier first to move the comments back to the same line
-  text.mask = gsub('(%\b%)[ ]*\n', '\\1', text.mask)
+  x = gsub('(%\b%)[ ]*\n', '\\1', x)
   # move 'else ...' back to the last line
-  text.mask = gsub('\n\\s*else(\\s+|$)', ' else\\1', text.mask)
-  if (any(grepl('\\\\\\\\', text.mask)) || any(grepl(inline.comment, text.mask))) {
-    m = gregexpr(inline.comment, text.mask)
-    regmatches(text.mask, m) = lapply(regmatches(text.mask, m), restore_bs)
+  x = gsub('\n\\s*else(\\s+|$)', ' else\\1', x)
+  if (any(grepl('\\\\\\\\', x)) || any(grepl(inline.comment, x))) {
+    m = gregexpr(inline.comment, x)
+    regmatches(x, m) = lapply(regmatches(x, m), restore_bs)
   }
-  # restore infix operators such as %>%
-  text.tidy = gsub(paste0('(%)(', infix_ops, ')', spaces, '(%)\\s*(\n)'), '\\1\\2\\3\\4', text.mask)
   # inline comments should be terminated by $ or \n
-  text.tidy = gsub(paste(inline.comment, '(\n|$)', sep = ''), '  \\1\\2', text.tidy)
+  x = gsub(paste(inline.comment, '(\n|$)', sep = ''), '  \\1\\2', x)
   # the rest of inline comments should be appended by \n
-  gsub(inline.comment, '  \\1\n', text.tidy)
+  gsub(inline.comment, '  \\1\n', x)
 }
 
 
@@ -238,3 +253,59 @@ tidy_file = function(file, ...) {
     try(tidy_source(f, file = f, ...))
   }
 }
+
+#' Reformat R code in RStudio IDE
+#'
+#' If any R code is selected in the RStudio source editor, this function
+#' reformats the selected code; otherwise it reformats the current open file (if
+#' it is unsaved, it will be automatically saved).
+#' @param ... Arguments to be passed to \code{\link{tidy_source}()}, among which
+#'   the \code{indent} argument will respect the value you set for the number of
+#'   spaces for indentation in RStudio.
+#' @note If the output is not what you want, you can undo the change in the
+#'   editor (Ctrl + Z or Command + Z).
+#' @export
+#' @examplesIf interactive()
+#' formatR::tidy_rstudio()
+#' formatR::tidy_rstudio(args.newline = TRUE)
+tidy_rstudio = function(...) {
+  ctx = rstudio_context()
+  if (is.null(getOption('formatR.indent'))) {
+    opts = options(formatR.indent = rstudioapi::readRStudioPreference('num_spaces_for_tab', 4))
+    on.exit(options(opts), add = TRUE)
+  }
+  if (length(ctx$selection) == 1 && !identical(txt <- ctx$selection[[1]]$text, '')) {
+    res = tidy_source(text = txt, output = FALSE, ...)$text.tidy
+    rstudioapi::modifyRange(ctx$selection[[1]]$range, one_string(res), ctx$id)
+  } else {
+    rstudioapi::documentSave(ctx$id)
+    res = tidy_source(ctx$path, output = FALSE, ...)$text.tidy
+    writeLines(enc2utf8(res), ctx$path, useBytes = TRUE)
+  }
+}
+
+rstudio_context = function() {
+  ctx = rstudioapi::getSourceEditorContext()
+  if (is.null(ctx)) stop('There is no open document in the RStudio source editor.')
+  ctx
+}
+
+#' Substitute the \pkg{magrittr} pipe with R's native pipe operator
+#'
+#' Parse the R code in the RStudio editor, identify \code{\%>\%}, and substitute
+#' with \code{|>}.
+#' @note Currently this function only works inside the RStudio IDE, and may be
+#'   extended in future to deal with arbitrary R code elsewhere.
+#' @export
+#' @examplesIf interactive()
+#' formatR::tidy
+tidy_pipe = function() {
+  ctx = rstudio_context()
+  d = parse_data(ctx$contents)
+  i = d$token == 'SPECIAL' & d$text == '%>%'
+  if (!any(i)) return(invisible())
+  d = d[i, c('line1', 'col1', 'line2', 'col2')]
+  d[, 4] = d[, 4] + 1
+  r = unname(as.list(as.data.frame(t(d))))
+  rstudioapi::modifyRange(r, '|>', ctx$id)
+}
